//
//  SwitchingViewController.swift
//  View Switcher
//
//  Created by Socratis Tornaritis on 6/27/15.
//  Copyright (c) 2015 Socratis Tornaritis. All rights reserved.
//

import UIKit

class SwitchingViewController: UIViewController {
   // two properties that identify each of the two content views.
   private var blueViewController: BlueViewController!
   private var yellowViewController: YellowViewController!
   
    override func viewDidLoad() {
        super.viewDidLoad()

      // we are going to load the blue view controller initially, but let's create it first.
      
      // create the blue view controlloer
      blueViewController = storyboard?.instantiateViewControllerWithIdentifier("Blue") as! BlueViewController
      
      // set its frame to be the same as the root controller's.
      blueViewController.view.frame = view.frame
      
      // now, switch to it; the blue that is.
      switchViewController(from: nil, to: blueViewController)
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
      
      // Release the unused view controller(s)
      if blueViewController != nil && blueViewController.view.superview == nil {
         // blue view not part of the view hierarchy, so remove its controller
         blueViewController = nil
      }
      if yellowViewController != nil && yellowViewController.view.superview == nil {
         // yellow view not part of the view hierarchy, so remove its controller
         yellowViewController = nil
      }
    }
    

    /*
    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        // Get the new view controller using segue.destinationViewController.
        // Pass the selected object to the new view controller.
    }
    */
   
   @IBAction func switchViews(sender: UIBarButtonItem) {
      // Create the new view controller, if required
      if yellowViewController?.view.superview == nil {
         // optional binding: if yellowViewController is nil because it is not created, create it.
         // if yellowViewController does exist, but is not currently loaded (superview == nil), it will be switched to further down
         // in the code.
         if yellowViewController == nil {
            yellowViewController = storyboard?.instantiateViewControllerWithIdentifier("Yellow") as! YellowViewController
         }
      } else if blueViewController?.view.superview == nil {
         // a) exists, but not loaded or b) does not exist.
         if blueViewController == nil {
            // the blue controller may have been flushed from memory, so create it if need be.
            blueViewController = storyboard?.instantiateViewControllerWithIdentifier("Blue") as! BlueViewController
         }
      }
      
      // Add some animation magic
      UIView.beginAnimations("View Flip", context: nil)
      UIView.setAnimationDuration(0.8)
      UIView.setAnimationCurve(.EaseInOut)
      
      // Swtich view controllers
      if blueViewController.view.superview != nil {
         // from blue to yellow
         UIView.setAnimationTransition(.FlipFromRight, forView: view, cache: true)
         
         yellowViewController.view.frame = view.frame
         switchViewController(from: blueViewController, to: yellowViewController)
      } else {
         // from yellow to blue
         UIView.setAnimationTransition(.FlipFromLeft, forView: view, cache: true)
         
         blueViewController.view.frame = view.frame
         switchViewController(from: yellowViewController, to: blueViewController)
      }
      UIView.commitAnimations()
   }

   private func switchViewController(from fromVC:UIViewController?, to toVC:UIViewController?) {
      // remove the controller we are switching from.
      if fromVC != nil {
         fromVC!.willMoveToParentViewController(nil)
         fromVC!.view.removeFromSuperview()
         fromVC!.removeFromParentViewController()
      }
      
      // add the controller we are switching to.
      if toVC != nil {
         // add the controller to the RC (root controller)
         self.addChildViewController(toVC!)
         
         // make this V the one in the very back.
         self.view.insertSubview(toVC!.view, atIndex: 0)   // backmost view
         
         // tell the C (controller) that is has been added as a child of another C.
         toVC!.didMoveToParentViewController(self)
      }
   }
}
