//
//  ViewController.swift
//  TipCalculator
//
//  Created by Socratis Tornaritis on 7/1/15.
//  Copyright (c) 2015 Socratis Tornaritis. All rights reserved.
//

import UIKit

class ViewController: UIViewController {
   

   @IBOutlet weak var billAmountLabel: UILabel!
   @IBOutlet weak var customTipPercentLabel: UILabel!
   @IBOutlet weak var customTipPercentSlider: UISlider!
   @IBOutlet weak var customTipPercentHeadingLabel: UILabel!
   @IBOutlet weak var tip15Label: UILabel!
   @IBOutlet weak var total15Label: UILabel!
   @IBOutlet weak var tipCustomLabel: UILabel!
   @IBOutlet weak var totalCustomLabel: UILabel!
   @IBOutlet weak var inputTextField: UITextField!
   
   // NSDecimalNumber constants used the calculateTip method.
   // Avoids the floating point rounding errors issue
   let decimal100 = NSDecimalNumber(string: "100.0")
   let decimal15Percent = NSDecimalNumber(string: "0.15")
   
   
   
   func updateCustomTipPercentLabelAndHeading() {
      // convert the slider value to an NSDecimalNumber
      let sliderValue = NSDecimalNumber(integer: Int(customTipPercentSlider.value))
      let customPercent = sliderValue / decimal100
      
      customTipPercentLabel.text = NSNumberFormatter.localizedStringFromNumber(customPercent, numberStyle: .PercentStyle)
      customTipPercentHeadingLabel.text = customTipPercentLabel.text
   }
   
   func updateBillAmountLabel() {
      let billAmount: NSDecimalNumber
      
      if inputTextField.text.isEmpty {
         billAmount = 0.0
      } else {
         let inputString = inputTextField.text
         billAmount = NSDecimalNumber(string: inputString) / decimal100
      }
      // update billAmountLabel with currency-formatted total
      billAmountLabel.text = " " + formatAsCurrency(billAmount)
   }
   
   func updateTipAndTotalAmounts() {
      let billAmount: NSDecimalNumber
      
      if inputTextField.text.isEmpty {
         billAmount = 0.0
      } else {
         billAmount = NSDecimalNumber(string: inputTextField.text) / decimal100
      }
      let customPercent = NSDecimalNumber(integer: Int(customTipPercentSlider.value)) / decimal100
      
      // calcuate and display the 15% tip and total
      let fifteenTip = billAmount * decimal15Percent
      
      tip15Label.text = formatAsCurrency(fifteenTip)
      total15Label.text = formatAsCurrency(billAmount + fifteenTip)
      
      // calculate custom tip and display custom tip and total
      let customTip = billAmount * customPercent
      
      tipCustomLabel.text = formatAsCurrency(customTip)
      totalCustomLabel.text = formatAsCurrency(billAmount + customTip)
   }
   
   func clearLabels() {
      // Clear the labels
      billAmountLabel.text = ""
      tip15Label.text = ""
      total15Label.text = ""
      tipCustomLabel.text = ""
      totalCustomLabel.text = ""
   }
   
   @IBAction func computeTip(sender: AnyObject) {
      // method gets called by both the slider and text field
      
      if sender is UISlider {
         // update the custom tip percent label and heading
         updateCustomTipPercentLabelAndHeading()
         
         let isBillShown = !inputTextField.text.isEmpty
         
         if isBillShown { updateTipAndTotalAmounts() }
         else           { clearLabels()              }
         
      } else if sender is UITextField {
         // update the bill amount label, tip and total amounts
         updateBillAmountLabel()
         updateTipAndTotalAmounts()
      }
   }// end computeTip()
   
   override func viewDidLoad() {
      super.viewDidLoad()
      // Make sure the keypad shows up immediately
      inputTextField.becomeFirstResponder()
   }
}

func formatAsCurrency(number: NSNumber) -> String {
   return NSNumberFormatter.localizedStringFromNumber(number, numberStyle: .CurrencyStyle)
}

func + (left: NSDecimalNumber, right: NSDecimalNumber) -> NSDecimalNumber {
   return left.decimalNumberByAdding(right)
}

func * (left: NSDecimalNumber, right: NSDecimalNumber) -> NSDecimalNumber {
   return left.decimalNumberByMultiplyingBy(right)
}

func / (left: NSDecimalNumber, right: NSDecimalNumber) -> NSDecimalNumber {
   return left.decimalNumberByDividingBy(right)
}
